package semorg.app;

import java.sql.SQLException; 

import org.eclipse.swt.widgets.Display;

import semorg.gui.MainWindow;
import semorg.gui.util.Configuration;
import semorg.sql.access.DBAccess;
import semorg.sql.util.UtilitySQL;

/**
 * This class manages the startup process of the application. This includes the
 * initilization of some utils and the database layer as well as the creation of
 * the main window of the application.
 * 
 */
public class SemOrg {

    /** Holds the configuration of the application. */
    private static Configuration huSemOrgConf;

    /**
     * Initializes the application by initializing the database connection and
     * some utilities.
     * 
     * @param createConfFileIfNotExists
     *                if <tt>true</tt> and a configuration file could not be
     *                found, it will be created. If <tt>false</tt> the
     *                configuration file will not be created.
     * 
     * @see semorg.sql.access.DBAccess
     * @see semorg.sql.util.UtilitySQL
     */
    public static void initializeApplication(boolean createConfFileIfNotExists)
	    throws ClassNotFoundException, SQLException {
	huSemOrgConf = new Configuration(createConfFileIfNotExists);
	DBAccess.init();
	UtilitySQL.initUtilitySQL();
    }

    /**
     * Returns a {@link Configuration} object which contains the actual
     * configuration.
     */
    public static Configuration getConfiguration() {
	return huSemOrgConf;
    }

    /**
     * The main function which implements the application starting process:
     * <ul>
     * <li>loading the configuration file or create one with the default
     * configuration if it does not exist.</li>
     * <li>initialization of the database connection including the creating of
     * the catalog and its tables if they does not exist.</li>
     * <li>creating a {@link MainWindow} instance. </li>
     * </ul>
     */
    public static void main(String[] args) {
	// make config
	try {
	    SemOrg.initializeApplication(true);
	} catch (ClassNotFoundException e) {
	    System.out
		    .println("The MySQL database driver was not found in the classpath.");
	    System.exit(1);
	} catch (SQLException e) {
	    e.printStackTrace();
	    System.exit(2);
	}
	new MainWindow(Display.getDefault());
    }

}
